@@ -215,6 +215,42 @@ class CBOM(models.Model):
                                     qty=cbomitem.qty, unit=cbomitem.unit))
         return bomitems
 
+    def get_csv_file(self):
+        """
+        Returns the url for a cBOM.csv file created from this CBOM instance.
+
+        """
+        file_name = 'cBOM_{0}.csv'.format(self.id)
+        path = os.path.join(settings.MEDIA_ROOT, file_name)
+        with open(path, 'w') as f:
+            csv_writer = DictWriter(f, app_settings.CBOM_CSV_FIELDNAMES)
+            csv_writer.writeheader()
+            clean_list = []
+            # for each bomitem...
+            for cbomitem in self.cbomitems.all():
+                # ... create a dictionary
+                cbomitem_dict = model_to_dict(cbomitem)
+                # ... clean the values
+                clean_dict = {}
+                for fieldname in app_settings.CBOM_CSV_FIELDNAMES:
+                    if fieldname.lower() == 'epn':
+                        clean_dict.update({fieldname: cbomitem.epn.epn})
+                    elif fieldname.lower() == 'consign':
+                        clean_dict.update({fieldname: int(cbomitem.consign)})
+                    elif fieldname.lower() == 'unit':
+                        clean_dict.update({fieldname: cbomitem.unit.code})
+                    elif fieldname.lower() == 'description':
+                        clean_dict.update({
+                            fieldname: cbomitem.epn.description})
+                    else:
+                        clean_dict.update({
+                            fieldname: cbomitem_dict[fieldname.lower()]})
+                # ... append the dict to the list
+                clean_list.append(clean_dict)
+            #... and write the entire list of dicts to the file
+            csv_writer.writerows(clean_list)
+        return os.path.join(settings.MEDIA_URL, file_name)
+
 
 class CBOMItem(models.Model):
     """
@@ -386,7 +422,6 @@ class IPN(models.Model):
     :name: A name for this IPN.
     :price_group: The price group, this IPN is in.
     :shape: TODO
-    :price_max: The maximum price.
 
     reverse relations:
     :additional_texts: The list of AdditionalText objects attached.
@@ -418,11 +453,14 @@ class IPN(models.Model):
         related_name='ipns',
     )
 
-    price_max = models.DecimalField(
-        verbose_name=_('Price max'),
-        max_digits=11,
-        decimal_places=3,
-    )
+    @property
+    def price_max(self):
+        try:
+            latest_marker = PriceMarker.objects.filter(ipn=self).latest('date')
+        except PriceMarker.DoesNotExist:
+            return
+        return latest_marker.price * self.price_group.rate + \
+            self.price_group.add
 
     def __unicode__(self):
         return u'{0} - IPN: {1}'.format(self.name, self.code)
@@ -461,7 +499,7 @@ class PriceGroup(models.Model):
     )
 
     def __unicode__(self):
-        return u'{0} - Rate: {1} - Add: {1}'.format(
+        return u'{0} = Price x {1} + {2}'.format(
             self.code, self.rate, self.add)
 
 
